require 'spec_helper'

describe Sudoku::GridFactory do
  GRID_FACTORY ||= Sudoku::GridFactory
  CELL_COORDINATES ||= Sudoku::CellCoordinates
  let(:factory) { GRID_FACTORY.new }

  before do
    factory.disable_log
  end

  it 'creates all coordinates set correctly' do
    set = factory.generate_all_coordinates
    expect(set.include?(CELL_COORDINATES.new(0, 0))).to be true
    expect(set.include?(CELL_COORDINATES.new(8, 8))).to be true
    expect(set.size).to eql(81)
  end

  context 'grid creation' do
    let(:random_grid) { factory.create_random }
    let(:solved_grid) { factory.create_full_grid }
    let(:free_cell) { Sudoku::Cell.new(CELL_COORDINATES.random, false, 5) }

    it 'the created solved grid does not have any free cells' do
      invalid_cells = solved_grid.cells.values.select { |x| !x.fixed }
      expect(invalid_cells.size).to eql(0)
    end

    it 'the random grid has no fixed cells with values other than 0' do
      invalid = random_grid.cells.values.select { |x| !x.fixed && x.value > 0 }
      expect(invalid.size).to eql(0)
    end

    it 'the random grid does not contains fixed cells with value = 0' do
      invalid = random_grid.cells.values.select { |x| x.fixed && x.value == 0 }
      expect(invalid.size).to eql(0)
    end

    it 'number of opened cells corresponds to the settings of factory' do
      cells = random_grid.cells.values.select { |x| !x.fixed }
      expect(cells.size).to eql(GRID_FACTORY::CELLS_TO_OPEN)
    end

    it 'parsing of bad sudoku format fails' do
      expect { factory.parse_file('lib/version.rb') }.to raise_error
    end
  end
end
