require 'spec_helper'

describe Sudoku::CellCoordinates do
  CELL_COORDINATES ||= Sudoku::CellCoordinates
  COORDINATE ||= Sudoku::Coordinate

  let(:cell_coordinates_5_5) { CELL_COORDINATES.new(5, 5) }
  let(:cell_coordinates_7_7) { CELL_COORDINATES.new(7, 7) }
  let(:cell_coordinates_3_7) { CELL_COORDINATES.new(COORDINATE.new(3), 7) }
  let(:cell_coordinates_wrong) { CELL_COORDINATES.new(0, 9) }

  it 'proper initializations' do
    expect(cell_coordinates_5_5.coordinate_x).to eql(5)
    expect(cell_coordinates_5_5.coordinate_y).to eql(5)
    expect(cell_coordinates_3_7.coordinate_x).to eql(3)
    expect(cell_coordinates_3_7.coordinate_y).to eql(7)
    expect { cell_coordinates_wrong }.to raise_error
  end

  it 'proper eql method' do
    expect(cell_coordinates_5_5).to eql(cell_coordinates_5_5)
    expect(cell_coordinates_5_5.eql?(CELL_COORDINATES.new(5, 5))).to be true
    expect(cell_coordinates_5_5).not_to eql(cell_coordinates_3_7)
    expect(cell_coordinates_5_5).not_to eql(5)
  end

  it 'proper to_s method' do
    expect(cell_coordinates_5_5.to_s).to eql('[5, 5]')
  end

  it 'proper hash method' do
    expect(cell_coordinates_5_5.hash).to eql(CELL_COORDINATES.new(5, 5).hash)
    expect(cell_coordinates_5_5.hash == cell_coordinates_3_7.hash).to be false
  end
end
